@@ -1,5 +1,6 @@
 from ._auth_app import AuthApp
 from ._core_app import CoreApp
+from ._data_app import DataApp
 from ._download_app import DownloadApp
 from ._dump_app import DumpApp
 from ._landing_app import LandingApp
@@ -15,6 +16,7 @@ from ._upload_app import UploadApp
 __all__ = [
     "AuthApp",
     "CoreApp",
+    "DataApp",
     "DownloadApp",
     "DumpApp",
     "LandingApp",
