@@ -16,7 +16,7 @@ from cognite.client.data_classes.data_modeling.instances import (
 
 from cognite_toolkit._cdf_tk.constants import COGNITE_MIGRATION_SPACE
 from cognite_toolkit._cdf_tk.tk_warnings import IgnoredValueWarning
-from cognite_toolkit._cdf_tk.utils.useful_types import AssetCentric
+from cognite_toolkit._cdf_tk.utils.useful_types import AssetCentricType
 
 if sys.version_info >= (3, 11):
     from typing import Self
@@ -26,7 +26,7 @@ else:
 
 @dataclass(frozen=True)
 class AssetCentricId(CogniteObject):
-    resource_type: AssetCentric
+    resource_type: AssetCentricType
     id_: int
 
     @classmethod
@@ -97,7 +97,7 @@ class InstanceSource(_InstanceSourceProperties, TypedNode):
         last_updated_time: int,
         created_time: int,
         *,
-        resource_type: Literal["asset", "event", "file", "sequence", "timeseries"],
+        resource_type: AssetCentricType,
         id_: int,
         data_set_id: int | None = None,
         classic_external_id: str | None = None,
@@ -151,6 +151,9 @@ class InstanceSource(_InstanceSourceProperties, TypedNode):
         }[self.resource_type]
         return ViewId("cdf_cdm", external_id, "v1")
 
+    def as_direct_relation_reference(self) -> DirectRelationReference:
+        return DirectRelationReference(space=self.space, external_id=self.external_id)
+
 
 class _ResourceViewMapping:
     resource_type = PropertyOptions("resourceType")
@@ -347,3 +350,100 @@ class ResourceViewMapping(_ResourceViewMapping, TypedNode):
             "viewId": self.view_id.dump(),
             "propertyMapping": self.property_mapping,
         }
+
+
+class CreatedSourceSystem(TypedNode):
+    """This represents the reading format of instance source.
+
+    It is used to when data is read from CDF.
+
+    The source of the instance in asset-centric resources.
+
+    Args:
+        space: The space where the node is located.
+        external_id: The external id of the instance source.
+        version (int): DMS version.
+        last_updated_time (int): The number of milliseconds since 00:00:00 Thursday, 1 January 1970,
+            Coordinated Universal Time (UTC), minus leap seconds.
+        created_time (int): The number of milliseconds since 00:00:00 Thursday, 1 January 1970,
+            Coordinated Universal Time (UTC), minus leap seconds.
+        source: The source string from the asset-centric Asset, Event, or FileMetadata source property.
+        type: Direct relation pointing to the type node.
+        deleted_time: The number of milliseconds since 00:00:00 Thursday, 1 January 1970, Coordinated Universal Time
+            (UTC), minus leap seconds. Timestamp when the instance was soft deleted. Note that deleted instances
+            are filtered out of query results, but present in sync results
+    """
+
+    def __init__(
+        self,
+        space: str,
+        external_id: str,
+        version: int,
+        last_updated_time: int,
+        created_time: int,
+        *,
+        source: str,
+        type: DirectRelationReference | None = None,
+        deleted_time: int | None = None,
+    ) -> None:
+        TypedNode.__init__(self, space, external_id, version, last_updated_time, created_time, deleted_time, type)
+        self.source = source
+
+    @classmethod
+    def get_source(cls) -> ViewId:
+        return ViewId("cognite_migration", "CreatedSourceSystem", "v1")
+
+    def as_direct_relation_reference(self) -> DirectRelationReference:
+        return DirectRelationReference(space=self.space, external_id=self.external_id)
+
+
+class SpaceSource(TypedNode):
+    """This represents the reading format of space source.
+
+    It is used to when data is read from CDF.
+
+    The source dataSet of an instance space
+
+    Args:
+        space: The space where the node is located.
+        external_id: The external id of the node.
+        version (int): DMS version.
+        last_updated_time (int): The number of milliseconds since 00:00:00 Thursday, 1 January 1970,
+            Coordinated Universal Time (UTC), minus leap seconds.
+        created_time (int): The number of milliseconds since 00:00:00 Thursday, 1 January 1970,
+            Coordinated Universal Time (UTC), minus leap seconds.
+        instance_space (str): The instance space name.
+        data_set_id: The data set id associated with the instance space.
+        data_set_external_id: The classic external id associated with the instance space.
+        type: Direct relation pointing to the type node.
+        deleted_time: The number of milliseconds since 00:00:00 Thursday, 1 January 1970, Coordinated Universal Time
+            (UTC), minus leap seconds. Timestamp when the instance was soft deleted. Note that deleted instances
+            are filtered out of query results, but present in sync results
+    """
+
+    instance_space = PropertyOptions("instanceSpace")
+    data_set_id = PropertyOptions("dataSetId")
+    data_set_external_id = PropertyOptions("dataSetExternalId")
+
+    def __init__(
+        self,
+        space: str,
+        external_id: str,
+        version: int,
+        last_updated_time: int,
+        created_time: int,
+        *,
+        instance_space: str,
+        data_set_id: int,
+        data_set_external_id: str | None = None,
+        type: DirectRelationReference | None = None,
+        deleted_time: int | None = None,
+    ) -> None:
+        TypedNode.__init__(self, space, external_id, version, last_updated_time, created_time, deleted_time, type)
+        self.instance_space = instance_space
+        self.data_set_id = data_set_id
+        self.data_set_external_id = data_set_external_id
+
+    @classmethod
+    def get_source(cls) -> ViewId:
+        return ViewId("cognite_migration", "SpaceSource", "v1")
