@@ -2,17 +2,17 @@ from collections.abc import Iterable
 from functools import partial
 from pathlib import Path
 
-from cognite.client.data_classes._base import T_CogniteResourceList
+from cognite.client.data_classes._base import T_CogniteResource
 from rich.console import Console
 
 from cognite_toolkit._cdf_tk.constants import DATA_MANIFEST_STEM, DATA_RESOURCE_DIR
 from cognite_toolkit._cdf_tk.exceptions import ToolkitValueError
-from cognite_toolkit._cdf_tk.storageio import ConfigurableStorageIO, StorageIO, T_Selector, TableStorageIO
+from cognite_toolkit._cdf_tk.storageio import ConfigurableStorageIO, Page, StorageIO, T_Selector, TableStorageIO
 from cognite_toolkit._cdf_tk.tk_warnings import LowSeverityWarning
 from cognite_toolkit._cdf_tk.utils.file import safe_write, sanitize_filename, yaml_safe_dump
 from cognite_toolkit._cdf_tk.utils.fileio import TABLE_WRITE_CLS_BY_FORMAT, Compression, FileWriter, SchemaColumn
 from cognite_toolkit._cdf_tk.utils.producer_worker import ProducerWorkerExecutor
-from cognite_toolkit._cdf_tk.utils.useful_types import T_ID, JsonVal, T_WritableCogniteResourceList
+from cognite_toolkit._cdf_tk.utils.useful_types import JsonVal
 
 from ._base import ToolkitCommand
 
@@ -21,7 +21,7 @@ class DownloadCommand(ToolkitCommand):
     def download(
         self,
         selectors: Iterable[T_Selector],
-        io: StorageIO[T_ID, T_Selector, T_CogniteResourceList, T_WritableCogniteResourceList],
+        io: StorageIO[T_Selector, T_CogniteResource],
         output_dir: Path,
         verbose: bool,
         file_format: str,
@@ -45,7 +45,7 @@ class DownloadCommand(ToolkitCommand):
         for selector in selectors:
             target_dir = output_dir / selector.group
             if verbose:
-                console.print(f"Downloading {io.DISPLAY_NAME} '{selector!s}' to {target_dir.as_posix()!r}")
+                console.print(f"Downloading {selector.display_name} '{selector!s}' to {target_dir.as_posix()!r}")
 
             iteration_count = self._get_iteration_count(io, selector, limit)
             filestem = sanitize_filename(str(selector))
@@ -62,15 +62,15 @@ class DownloadCommand(ToolkitCommand):
                 columns = io.get_schema(selector)
             elif file_format in TABLE_WRITE_CLS_BY_FORMAT:
                 raise ToolkitValueError(
-                    f"Cannot download {io.KIND} in {file_format!r} format. The {io.KIND!r} storage type does not support table schemas."
+                    f"Cannot download {selector.kind} in {file_format!r} format. The {selector.kind!r} storage type does not support table schemas."
                 )
 
             with FileWriter.create_from_format(
-                file_format, target_dir, io.KIND, compression_cls, columns=columns
+                file_format, target_dir, selector.kind, compression_cls, columns=columns
             ) as writer:
-                executor = ProducerWorkerExecutor[T_WritableCogniteResourceList, list[dict[str, JsonVal]]](
+                executor = ProducerWorkerExecutor[Page[T_CogniteResource], list[dict[str, JsonVal]]](
                     download_iterable=io.stream_data(selector, limit),
-                    process=io.data_to_json_chunk,
+                    process=partial(self.process_data_chunk, io=io, selector=selector),
                     write=partial(writer.write_chunks, filestem=filestem),
                     iteration_count=iteration_count,
                     # Limit queue size to avoid filling up memory before the workers can write to disk.
@@ -95,7 +95,7 @@ class DownloadCommand(ToolkitCommand):
 
     @staticmethod
     def _get_iteration_count(
-        io: StorageIO[T_ID, T_Selector, T_CogniteResourceList, T_WritableCogniteResourceList],
+        io: StorageIO[T_Selector, T_CogniteResource],
         selector: T_Selector,
         limit: int | None,
     ) -> int | None:
@@ -123,3 +123,21 @@ class DownloadCommand(ToolkitCommand):
                 return True
 
         return False
+
+    @staticmethod
+    def process_data_chunk(
+        data_page: Page[T_CogniteResource],
+        io: StorageIO[T_Selector, T_CogniteResource],
+        selector: T_Selector,
+    ) -> list[dict[str, JsonVal]]:
+        """Processes a chunk of data by converting it to a JSON-compatible format.
+
+        Args:
+            data_page: The page of data to process.
+            io: The StorageIO instance that defines how to process the data.
+            selector: The selection criteria used to identify the data.
+
+        Returns:
+            A list of dictionaries representing the processed data in a JSON-compatible format.
+        """
+        return io.data_to_json_chunk(data_page.items, selector)
