@@ -12,9 +12,10 @@ from cognite_toolkit._cdf_tk.exceptions import (
     ToolkitValueError,
 )
 from cognite_toolkit._cdf_tk.storageio._data_classes import ModelList
+from cognite_toolkit._cdf_tk.utils.useful_types import AssetCentricType
 
 
-class MigrationMapping(BaseModel, alias_generator=to_camel_case, extra="ignore"):
+class MigrationMapping(BaseModel, alias_generator=to_camel_case, extra="ignore", populate_by_name=True):
     """The mapping between an asset-centric ID and a data modeling instance ID.
     Args
         resource_type (str): The asset-centric type of the resource (e.g., "asset", "event", "timeseries").
@@ -27,7 +28,7 @@ class MigrationMapping(BaseModel, alias_generator=to_camel_case, extra="ignore")
            for example, the Canvas migration to determine which view to use for the resource.
     """
 
-    resource_type: str
+    resource_type: AssetCentricType
     instance_id: NodeId
     id: int
     data_set_id: int | None = None
@@ -46,7 +47,7 @@ class MigrationMapping(BaseModel, alias_generator=to_camel_case, extra="ignore")
         raise ToolkitValueError(f"No default ingestion view specified for resource type '{self.resource_type}'")
 
     def as_asset_centric_id(self) -> AssetCentricId:
-        return AssetCentricId(resource_type=self.resource_type, id_=self.id)  # type: ignore[arg-type]
+        return AssetCentricId(resource_type=self.resource_type, id_=self.id)
 
     @model_validator(mode="before")
     def _handle_flat_dict(cls, values: Any) -> Any:
@@ -128,6 +129,7 @@ class MigrationMappingList(ModelList[MigrationMapping]):
             "asset": AssetMigrationMappingList,
             "timeseries": TimeSeriesMigrationMappingList,
             "file": FileMigrationMappingList,
+            "event": EventMigrationMappingList,
         }
         if resource_type not in cls_by_resource_type:
             raise ToolkitValueError(
@@ -140,6 +142,10 @@ class AssetMapping(MigrationMapping):
     resource_type: Literal["asset"] = "asset"
 
 
+class EventMapping(MigrationMapping):
+    resource_type: Literal["event"] = "event"
+
+
 class TimeSeriesMapping(MigrationMapping):
     resource_type: Literal["timeseries"] = "timeseries"
 
@@ -154,6 +160,12 @@ class AssetMigrationMappingList(MigrationMappingList):
         return AssetMapping
 
 
+class EventMigrationMappingList(MigrationMappingList):
+    @classmethod
+    def _get_base_model_cls(cls) -> type[EventMapping]:
+        return EventMapping
+
+
 class FileMigrationMappingList(MigrationMappingList):
     @classmethod
     def _get_base_model_cls(cls) -> type[FileMapping]:
