@@ -46,7 +46,7 @@ class MigrateFilesCommand(BaseMigrateCommand):
         auto_yes: bool = False,
     ) -> None:
         """Migrate resources from Asset-Centric to data modeling in CDF."""
-        mappings = MigrationMappingList.read_mapping_file(mapping_file, "file")
+        mappings = MigrationMappingList.read_csv_file(mapping_file, "file")
         self.validate_access(
             client,
             instance_spaces=list(mappings.spaces()),
