@@ -4,8 +4,17 @@ from cognite_toolkit._cdf_tk.utils._auxiliary import get_concrete_subclasses
 from cognite_toolkit._cdf_tk.utils.fileio import COMPRESSION_BY_SUFFIX
 
 from ._applications import CanvasIO, ChartIO
-from ._asset_centric import AssetIO, BaseAssetCentricIO, EventIO, FileMetadataIO, TimeSeriesIO
-from ._base import ConfigurableStorageIO, StorageIO, StorageIOConfig, T_Selector, TableStorageIO
+from ._asset_centric import AssetIO, BaseAssetCentricIO, EventIO, FileMetadataIO, HierarchyIO, TimeSeriesIO
+from ._base import (
+    ConfigurableStorageIO,
+    Page,
+    StorageIO,
+    StorageIOConfig,
+    T_Selector,
+    TableStorageIO,
+    UploadableStorageIO,
+    UploadItem,
+)
 from ._data_classes import InstanceIdCSVList, InstanceIdRow, ModelList
 from ._instances import InstanceIO
 from ._raw import RawIO
@@ -13,14 +22,15 @@ from .selectors._base import DataSelector
 
 # MyPy fails to recognize that get_concrete_subclasses always returns a list of concrete subclasses.
 STORAGE_IO_CLASSES = get_concrete_subclasses(StorageIO)  # type: ignore[type-abstract]
+UPLOAD_IO_CLASSES = get_concrete_subclasses(UploadableStorageIO)  # type: ignore[type-abstract]
 
 
-def get_storage_io(selector_cls: type[DataSelector], kind: str | Path) -> type[StorageIO]:
-    """Get the appropriate StorageIO class based on the type of the provided selector."""
-    for cls in STORAGE_IO_CLASSES:
+def get_upload_io(selector_cls: type[DataSelector], kind: str | Path) -> type[UploadableStorageIO]:
+    """Get the appropriate UploadableStorageIO class based on the type of the provided selector."""
+    for cls in UPLOAD_IO_CLASSES:
         if issubclass(selector_cls, cls.BASE_SELECTOR) and are_same_kind(cls.KIND, kind):
             return cls
-    raise ValueError(f"No StorageIO found for selector of type {selector_cls.__name__}")
+    raise ValueError(f"No UploadableStorageIO found for selector of type {selector_cls.__name__}")
 
 
 def are_same_kind(kind: str, kind_or_path: str | Path, /) -> bool:
@@ -41,16 +51,20 @@ __all__ = [
     "ConfigurableStorageIO",
     "EventIO",
     "FileMetadataIO",
+    "HierarchyIO",
     "InstanceIO",
     "InstanceIdCSVList",
     "InstanceIdRow",
     "ModelList",
+    "Page",
     "RawIO",
     "StorageIO",
     "StorageIOConfig",
     "T_Selector",
     "TableStorageIO",
     "TimeSeriesIO",
+    "UploadItem",
+    "UploadableStorageIO",
     "are_same_kind",
-    "get_storage_io",
+    "get_upload_io",
 ]
