@@ -1,49 +1,39 @@
 from ._client import HTTPClient
 from ._data_classes import (
-    FailedItem,
-    FailedRequestItem,
+    ErrorDetails,
+    FailedRequestItems,
     FailedRequestMessage,
     FailedResponse,
+    FailedResponseItems,
     HTTPMessage,
-    ItemIDMessage,
     ItemMessage,
-    ItemResponse,
     ItemsRequest,
-    MissingItem,
     ParamRequest,
     RequestMessage,
     ResponseList,
     ResponseMessage,
     SimpleBodyRequest,
-    SuccessItem,
     SuccessResponse,
-    UnexpectedItem,
-    UnknownRequestItem,
-    UnknownResponseItem,
+    SuccessResponseItems,
 )
 from ._exception import ToolkitAPIError
 
 __all__ = [
-    "FailedItem",
-    "FailedRequestItem",
+    "ErrorDetails",
+    "FailedRequestItems",
     "FailedRequestMessage",
     "FailedResponse",
+    "FailedResponseItems",
     "HTTPClient",
     "HTTPMessage",
-    "ItemIDMessage",
     "ItemMessage",
-    "ItemResponse",
     "ItemsRequest",
-    "MissingItem",
     "ParamRequest",
     "RequestMessage",
     "ResponseList",
     "ResponseMessage",
     "SimpleBodyRequest",
-    "SuccessItem",
     "SuccessResponse",
+    "SuccessResponseItems",
     "ToolkitAPIError",
-    "UnexpectedItem",
-    "UnknownRequestItem",
-    "UnknownResponseItem",
 ]
