@@ -10,7 +10,6 @@ from cognite.client.exceptions import CogniteAPIError
 
 from cognite_toolkit._cdf_tk.client import ToolkitClient
 from cognite_toolkit._cdf_tk.client.data_classes.extended_timeseries import ExtendedTimeSeriesList
-from cognite_toolkit._cdf_tk.client.data_classes.migration import AssetCentricId
 from cognite_toolkit._cdf_tk.commands import MigrateFilesCommand
 from cognite_toolkit._cdf_tk.commands._migrate.adapter import (
     FileMetaAdapter,
@@ -124,14 +123,14 @@ class TestMigrateFilesCommand:
 
         cmd = MigrationCommand(skip_tracking=True, silent=True)
         results = cmd.migrate(
-            selected=MigrationCSVFileSelector(datafile=input_file, resource_type="file"),
-            data=FileMetaAdapter(client),
+            selected=MigrationCSVFileSelector(datafile=input_file, kind="file"),
+            data=FileMetaAdapter(client, skip_linking=False),
             mapper=AssetCentricMapper(client),
             log_dir=tmp_path / "logs",
             dry_run=False,
             verbose=False,
         )
-        actual_results = [results.get_progress(AssetCentricId("file", item.id)) for item in three_files_with_content]
+        actual_results = [results.get_progress(f"file_{item.id}") for item in three_files_with_content]
         expected_results = [
             {
                 cmd.Steps.DOWNLOAD: "success",
