@@ -23,7 +23,6 @@ from cognite.client.data_classes.data_modeling import (
 from cognite.client.data_classes.data_modeling.statistics import InstanceStatistics, ProjectStatistics
 
 from cognite_toolkit._cdf_tk.client import ToolkitClient, ToolkitClientConfig
-from cognite_toolkit._cdf_tk.client.data_classes.migration import AssetCentricId
 from cognite_toolkit._cdf_tk.client.testing import monkeypatch_toolkit_client
 from cognite_toolkit._cdf_tk.commands._migrate.adapter import AssetCentricMigrationIOAdapter, MigrationCSVFileSelector
 from cognite_toolkit._cdf_tk.commands._migrate.command import MigrationCommand
@@ -110,8 +109,57 @@ def cognite_migration_model(
     yield rsps
 
 
+@pytest.fixture
+def mock_statistics(
+    toolkit_config: ToolkitClientConfig,
+    rsps: responses.RequestsMock,
+) -> Iterator[responses.RequestsMock]:
+    config = toolkit_config
+    stats_response = {
+        "spaces": {
+            "count": 0,
+            "limit": 1_000,
+        },
+        "containers": {
+            "count": 0,
+            "limit": 10_000,
+        },
+        "views": {
+            "count": 0,
+            "limit": 100_000,
+        },
+        "dataModels": {
+            "count": 1,
+            "limit": 10_000,
+        },
+        "containerProperties": {
+            "count": 0,
+            "limit": 1_000_000,
+        },
+        "instances": {
+            "nodes": 1000,
+            "edges": 0,
+            "softDeletedNodes": 0,
+            "softDeletedEdges": 0,
+            "instancesLimit": 5_000_000,
+            "softDeletedInstancesLimit": 100_000_000,
+            "instances": 1000,
+            "softDeletedInstances": 0,
+        },
+        "concurrentReadLimit": 50,
+        "concurrentWriteLimit": 20,
+        "concurrentDeleteLimit": 10,
+    }
+    rsps.get(
+        config.create_api_url("/models/statistics"),
+        json=stats_response,
+        status=200,
+    )
+    yield rsps
+
+
 class TestMigrationCommand:
-    @pytest.mark.usefixtures("disable_gzip", "disable_pypi_check")
+    @pytest.mark.usefixtures("disable_gzip", "disable_pypi_check", "mock_statistics")
     def test_migrate_assets(
         self,
         toolkit_config: ToolkitClientConfig,
@@ -146,6 +194,12 @@ class TestMigrationCommand:
             json={"items": [asset.dump() for asset in assets]},
             status=200,
         )
+        # Lookup CogniteSourceSystem (no source systems defined)
+        rsps.post(
+            config.create_api_url("/models/instances/list"),
+            json={"items": []},
+            status=200,
+        )
         # Instance creation
         respx.post(
             config.create_api_url("/models/instances"),
@@ -168,7 +222,6 @@ class TestMigrationCommand:
                 },
             )
         )
-
         csv_file = tmp_path / "migration.csv"
         csv_file.write_text(csv_content, encoding="utf-8")
 
@@ -176,7 +229,7 @@ class TestMigrationCommand:
         command = MigrationCommand(silent=True)
 
         result = command.migrate(
-            selected=MigrationCSVFileSelector(datafile=csv_file, resource_type="asset"),
+            selected=MigrationCSVFileSelector(datafile=csv_file, kind="asset"),
             data=AssetCentricMigrationIOAdapter(client, AssetIO(client)),
             mapper=AssetCentricMapper(client),
             log_dir=tmp_path / "logs",
@@ -215,14 +268,14 @@ class TestMigrationCommand:
             for asset in assets
         ]
         assert actual_instances == expected_instance
-        actual_results = [result.get_progress(AssetCentricId("asset", asset.id)) for asset in assets]
+        actual_results = [result.get_progress(f"asset_{asset.id}") for asset in assets]
         expected_results = [{"download": "success", "convert": "success", "upload": "success"} for _ in assets]
         assert actual_results == expected_results
         csv_file = next((tmp_path / "logs").glob("*.csv"), None)
         assert csv_file is not None, "Expected a CSV log file to be created"
         csv_results = list(CSVReader(csv_file).read_chunks_unprocessed())
         assert csv_results == [
-            {"ID": f"asset(id={asset.id})", "download": "success", "convert": "success", "upload": "success"}
+            {"ID": f"asset_{asset.id}", "download": "success", "convert": "success", "upload": "success"}
             for asset in assets
         ]
 
