@@ -5,6 +5,9 @@ import pytest
 from cognite.client.data_classes import Asset
 from cognite.client.data_classes.data_modeling import (
     ContainerId,
+    DirectRelation,
+    DirectRelationReference,
+    InstanceApply,
     MappedProperty,
     NodeId,
     NodeList,
@@ -13,7 +16,7 @@ from cognite.client.data_classes.data_modeling import (
     ViewId,
 )
 
-from cognite_toolkit._cdf_tk.client.data_classes.migration import ResourceViewMapping
+from cognite_toolkit._cdf_tk.client.data_classes.migration import CreatedSourceSystem, ResourceViewMapping
 from cognite_toolkit._cdf_tk.client.testing import monkeypatch_toolkit_client
 from cognite_toolkit._cdf_tk.commands._migrate.adapter import (
     AssetCentricMapping,
@@ -22,7 +25,7 @@ from cognite_toolkit._cdf_tk.commands._migrate.adapter import (
 )
 from cognite_toolkit._cdf_tk.commands._migrate.data_classes import MigrationMapping
 from cognite_toolkit._cdf_tk.commands._migrate.data_mapper import AssetCentricMapper
-from cognite_toolkit._cdf_tk.commands._migrate.issues import ConversionIssue
+from cognite_toolkit._cdf_tk.commands._migrate.issues import ConversionIssue, MigrationIssue
 from cognite_toolkit._cdf_tk.exceptions import ToolkitValueError
 
 
@@ -41,6 +44,7 @@ class TestAssetCentricMapper:
                     resource=Asset(
                         id=1000 + i,
                         name=f"Asset {i}",
+                        source="sap",
                         # Half of the assets will be missing description and thus have a conversion issue.
                         description=f"Description {i}" if i % 2 == 0 else None,
                     ),
@@ -54,7 +58,7 @@ class TestAssetCentricMapper:
             + "\n".join(f"{1000 + i},my_space,asset_{i},cdf_asset_mapping" for i in range(asset_count))
         )
 
-        selected = MigrationCSVFileSelector(datafile=mapping_file, resource_type="asset")
+        selected = MigrationCSVFileSelector(datafile=mapping_file, kind="asset")
 
         with monkeypatch_toolkit_client() as client:
             client.migration.resource_view_mapping.retrieve.return_value = NodeList[ResourceViewMapping](
@@ -66,6 +70,7 @@ class TestAssetCentricMapper:
                         property_mapping={
                             "name": "name",
                             "description": "description",
+                            "source": "source",
                         },
                         last_updated_time=1,
                         created_time=0,
@@ -73,6 +78,18 @@ class TestAssetCentricMapper:
                     )
                 ]
             )
+            client.migration.created_source_system.list.return_value = NodeList[CreatedSourceSystem](
+                [
+                    CreatedSourceSystem(
+                        space="source_systems",
+                        external_id="SAP",
+                        source="sap",
+                        last_updated_time=1,
+                        created_time=0,
+                        version=1,
+                    ),
+                ]
+            )
             # Mocking the view to avoid setting all properties we don't use
             cognite_asset = MagicMock(spec=View)
             cognite_asset.properties = {
@@ -82,6 +99,9 @@ class TestAssetCentricMapper:
                 "description": MappedProperty(
                     ContainerId("cdf_cdm", "CogniteDescribable"), "description", Text(), True, False, False
                 ),
+                "source": MappedProperty(
+                    ContainerId("cdf_cdm", "CogniteSourceable"), "source", DirectRelation(False), True, False, False
+                ),
             }
             cognite_asset.as_id.return_value = ViewId("cdf_cdm", "CogniteAsset", "v1")
             client.data_modeling.views.retrieve.return_value = [cognite_asset]
@@ -90,7 +110,13 @@ class TestAssetCentricMapper:
 
             mapper.prepare(selected)
 
-            mapped, issues = mapper.map_chunk(source)
+            mapped: list[InstanceApply] = []
+            issues: list[MigrationIssue] = []
+            for item in source:
+                target, item_issue = mapper.map(item)
+                mapped.append(target)
+                if not isinstance(item_issue, ConversionIssue) or item_issue.has_issues:
+                    issues.append(item_issue)
 
             # We do not assert the exact content of mapped, as that is tested in the
             # tests for the asset_centric_to_dm function.
@@ -100,30 +126,29 @@ class TestAssetCentricMapper:
             first_issue = issues[0]
             assert isinstance(first_issue, ConversionIssue)
             assert first_issue.missing_asset_centric_properties == ["description"]
+            first_asset = mapped[0]
+            assert first_asset.sources[0].properties["source"] == DirectRelationReference("source_systems", "SAP")
 
             assert client.migration.resource_view_mapping.retrieve.call_count == 1
             client.migration.resource_view_mapping.retrieve.assert_called_with(["cdf_asset_mapping"])
+            assert client.migration.created_source_system.list.call_count == 1
             assert client.data_modeling.views.retrieve.call_count == 1
             client.data_modeling.views.retrieve.assert_called_with([ViewId("cdf_cdm", "CogniteAsset", "v1")])
 
     def test_map_chunk_before_prepare_raises_error(self, tmp_path: Path) -> None:
         """Test that calling map_chunk before prepare raises a RuntimeError."""
-        source = AssetCentricMappingList(
-            [
-                AssetCentricMapping(
-                    mapping=MigrationMapping(
-                        resourceType="asset",
-                        instanceId=NodeId(space="my_space", external_id="asset_1"),
-                        id=1001,
-                        ingestionView="cdf_asset_mapping",
-                    ),
-                    resource=Asset(
-                        id=1001,
-                        name="Asset 1",
-                        description="Description 1",
-                    ),
-                )
-            ]
+        source = AssetCentricMapping(
+            mapping=MigrationMapping(
+                resourceType="asset",
+                instanceId=NodeId(space="my_space", external_id="asset_1"),
+                id=1001,
+                ingestionView="cdf_asset_mapping",
+            ),
+            resource=Asset(
+                id=1001,
+                name="Asset 1",
+                description="Description 1",
+            ),
         )
 
         with monkeypatch_toolkit_client() as client:
@@ -134,14 +159,14 @@ class TestAssetCentricMapper:
                 RuntimeError,
                 match=r"Failed to lookup mapping or view for ingestion view 'cdf_asset_mapping'. Did you forget to call .prepare()?",
             ):
-                mapper.map_chunk(source)
+                mapper.map(source)
 
     def test_prepare_missing_view_source_raises_error(self, tmp_path: Path) -> None:
         """Test that prepare raises ToolkitValueError when view source is not found."""
         mapping_file = tmp_path / "mapping.csv"
         mapping_file.write_text("id,space,externalId,ingestionView\n1001,my_space,asset_1,missing_view_source")
 
-        selected = MigrationCSVFileSelector(datafile=mapping_file, resource_type="asset")
+        selected = MigrationCSVFileSelector(datafile=mapping_file, kind="asset")
 
         with monkeypatch_toolkit_client() as client:
             # Return empty list to simulate missing view source
@@ -159,7 +184,7 @@ class TestAssetCentricMapper:
         mapping_file = tmp_path / "mapping.csv"
         mapping_file.write_text("id,space,externalId,ingestionView\n1001,my_space,asset_1,cdf_asset_mapping")
 
-        selected = MigrationCSVFileSelector(datafile=mapping_file, resource_type="asset")
+        selected = MigrationCSVFileSelector(datafile=mapping_file, kind="asset")
 
         with monkeypatch_toolkit_client() as client:
             # Return view source but empty view list to simulate missing view in Data Modeling
