@@ -1,6 +1,7 @@
 import json
 from collections import Counter
 from collections.abc import Iterator
+from dataclasses import dataclass
 from unittest.mock import patch
 
 import httpx
@@ -10,21 +11,19 @@ import respx
 from cognite_toolkit._cdf_tk.client import ToolkitClientConfig
 from cognite_toolkit._cdf_tk.utils._auxiliary import get_concrete_subclasses
 from cognite_toolkit._cdf_tk.utils.http_client import (
-    FailedItem,
-    FailedRequestItem,
+    ErrorDetails,
+    FailedRequestItems,
     FailedRequestMessage,
     FailedResponse,
+    FailedResponseItems,
     HTTPClient,
     HTTPMessage,
+    ItemMessage,
     ItemsRequest,
-    MissingItem,
     ParamRequest,
     SimpleBodyRequest,
-    SuccessItem,
     SuccessResponse,
-    UnexpectedItem,
-    UnknownRequestItem,
-    UnknownResponseItem,
+    SuccessResponseItems,
 )
 from cognite_toolkit._cdf_tk.utils.useful_types import JsonVal
 from tests.test_unit.utils import FakeCogniteResourceGenerator
@@ -48,6 +47,7 @@ def http_client_one_retry(toolkit_config: ToolkitClientConfig) -> Iterator[HTTPC
         yield client
 
 
+@pytest.mark.usefixtures("disable_pypi_check")
 class TestHTTPClient:
     def test_get_request(self, rsps: respx.MockRouter, http_client: HTTPClient) -> None:
         rsps.get("https://example.com/api/resource").respond(json={"key": "value"}, status_code=200)
@@ -58,7 +58,7 @@ class TestHTTPClient:
         response = results[0]
         assert isinstance(response, SuccessResponse)
         assert response.status_code == 200
-        assert response.body == {"key": "value"}
+        assert response.body == '{"key":"value"}'
         assert rsps.calls[-1].request.url == "https://example.com/api/resource?query=test"
 
     @pytest.mark.usefixtures("disable_gzip")
@@ -73,7 +73,7 @@ class TestHTTPClient:
         response = results[0]
         assert isinstance(response, SuccessResponse)
         assert response.status_code == 201
-        assert response.body == {"id": 123, "status": "created"}
+        assert response.body == '{"id":123,"status":"created"}'
         assert rsps.calls[-1].request.content == json.dumps({"name": "new resource"}).encode()
 
     @pytest.mark.usefixtures("disable_gzip")
@@ -110,7 +110,7 @@ class TestHTTPClient:
         response = results[0]
         assert isinstance(response, FailedResponse)
         assert response.status_code == 400
-        assert response.error == "bad request"
+        assert response.error.message == "bad request"
 
     @pytest.mark.usefixtures("disable_gzip")
     def test_retry_then_success(self, rsps: respx.MockRouter, http_client: HTTPClient) -> None:
@@ -122,13 +122,13 @@ class TestHTTPClient:
         response = results[0]
         assert isinstance(response, SuccessResponse)
         assert response.status_code == 200
-        assert response.body == {"key": "value"}
+        assert response.body == '{"key":"value"}'
 
     def test_retry_exhausted(self, http_client_one_retry: HTTPClient, rsps: respx.MockRouter) -> None:
         client = http_client_one_retry
         for _ in range(2):
             rsps.get("https://example.com/api/resource").respond(
-                json={"error": {"message": "service unavailable"}}, status_code=503
+                json={"error": {"message": "service unavailable", "code": 503}}, status_code=503
             )
         with patch("time.sleep"):  # Patch sleep to speed up the test
             results = client.request_with_retries(
@@ -139,25 +139,18 @@ class TestHTTPClient:
         response = results[0]
         assert isinstance(response, FailedResponse)
         assert response.status_code == 503
-        assert response.error == "service unavailable"
-
-    def test_invalid_json_response(self, rsps: respx.MockRouter, http_client: HTTPClient) -> None:
-        rsps.get("https://example.com/api/resource").respond(content="not json", status_code=200)
-        results = http_client.request(ParamRequest(endpoint_url="https://example.com/api/resource", method="GET"))
-        assert len(results) == 1
-        response = results[0]
-        assert isinstance(response, FailedResponse)
-        assert response.status_code == 200
-        assert "Invalid JSON response" in response.error
+        assert response.error.message == "service unavailable"
 
     def test_connection_error(self, http_client_one_retry: HTTPClient, rsps: respx.MockRouter) -> None:
         http_client = http_client_one_retry
         rsps.get("http://nonexistent.domain/api/resource").mock(
             side_effect=httpx.ConnectError("Simulated connection error")
         )
-        results = http_client.request_with_retries(
-            ParamRequest(endpoint_url="http://nonexistent.domain/api/resource", method="GET")
-        )
+        with patch(f"{HTTPClient.__module__}.time"):
+            # Patch time to avoid actual sleep
+            results = http_client.request_with_retries(
+                ParamRequest(endpoint_url="http://nonexistent.domain/api/resource", method="GET")
+            )
         response = results[0]
         assert len(results) == 1
         assert isinstance(response, FailedRequestMessage)
@@ -167,7 +160,9 @@ class TestHTTPClient:
         http_client = http_client_one_retry
         rsps.get("https://example.com/api/resource").mock(side_effect=httpx.ReadTimeout("Simulated read timeout"))
         bad_request = ParamRequest(endpoint_url="https://example.com/api/resource", method="GET")
-        results = http_client.request_with_retries(bad_request)
+        with patch(f"{HTTPClient.__module__}.time"):
+            # Patch time to avoid actual sleep
+            results = http_client.request_with_retries(bad_request)
         response = results[0]
         assert len(results) == 1
         assert isinstance(response, FailedRequestMessage)
@@ -183,7 +178,7 @@ class TestHTTPClient:
         response = results[0]
         assert isinstance(response, FailedResponse)
         assert response.status_code == 503
-        assert response.error == "service unavailable"
+        assert response.error.message == "service unavailable"
         assert len(rsps.calls) == 1
 
     def test_raise_if_already_retied(self, http_client_one_retry: HTTPClient) -> None:
@@ -199,7 +194,7 @@ class TestHTTPClient:
         response = results[0]
         assert isinstance(response, FailedResponse)
         assert response.status_code == 401
-        assert response.error == '{"message":"plain_text"}'
+        assert response.error.message == '{"message":"plain_text"}'
 
     def test_request_alpha(self, http_client: HTTPClient, rsps: respx.MockRouter) -> None:
         rsps.get("https://example.com/api/alpha/endpoint").respond(json={"key": "value"}, status_code=200)
@@ -218,6 +213,28 @@ class TestHTTPClient:
         assert rsps.calls[-1].request.headers["cdf-version"] == "alpha"
 
 
+@dataclass
+class MyRequestItemID:
+    item: dict[str, JsonVal]
+
+    def dump(self) -> JsonVal:
+        return self.item
+
+    def as_id(self) -> int:
+        return self.item["id"]
+
+
+@dataclass
+class MyRequestItemExternalID:
+    item: dict[str, JsonVal]
+
+    def dump(self) -> JsonVal:
+        return self.item
+
+    def as_id(self) -> str:
+        return self.item["externalId"]
+
+
 @pytest.mark.usefixtures("disable_pypi_check")
 class TestHTTPClientItemRequests:
     @pytest.mark.usefixtures("disable_gzip")
@@ -226,19 +243,19 @@ class TestHTTPClientItemRequests:
             json={"items": [{"id": 1, "value": 42}, {"id": 2, "value": 43}]},
             status_code=200,
         )
-        items = [{"name": "item1", "id": 1}, {"name": "item2", "id": 2}]
+        items = [MyRequestItemID({"name": "item1", "id": 1}), MyRequestItemID({"name": "item2", "id": 2})]
         results = http_client.request(
             ItemsRequest[int](
                 endpoint_url="https://example.com/api/resource",
                 method="POST",
                 items=items,
                 extra_body_fields={"autoCreateDirectRelations": True},
-                as_id=lambda item: item["id"],
             )
         )
         assert results == [
-            SuccessItem(status_code=200, id=1, item={"id": 1, "value": 42}),
-            SuccessItem(status_code=200, id=2, item={"id": 2, "value": 43}),
+            SuccessResponseItems(
+                status_code=200, ids=[1, 2], body='{"items":[{"id":1,"value":42},{"id":2,"value":43}]}'
+            )
         ]
         assert len(rsps.calls) == 1
         assert json.loads(rsps.calls[0].request.content) == {
@@ -250,7 +267,6 @@ class TestHTTPClientItemRequests:
     def test_request_with_items_issues(self, http_client: HTTPClient, rsps: respx.MockRouter) -> None:
         response_items = [
             {"externalId": "success", "data": 123},
-            {"externalId": "unexpected", "data": 999},
         ]
 
         def server_callback(request: httpx.Request) -> httpx.Response:
@@ -266,123 +282,74 @@ class TestHTTPClientItemRequests:
         rsps.post("https://example.com/api/resource").mock(side_effect=server_callback)
 
         request_items = [
-            {"externalId": "success"},
-            {"externalId": "missing"},
-            {"externalId": "fail"},
+            MyRequestItemExternalID({"externalId": "success"}),
+            MyRequestItemExternalID({"externalId": "fail"}),
         ]
         results = http_client.request_with_retries(
             ItemsRequest[str](
                 endpoint_url="https://example.com/api/resource",
                 method="POST",
                 items=request_items,
-                as_id=lambda item: item["externalId"],
             )
         )
         assert results == [
-            SuccessItem(status_code=200, id="success", item={"externalId": "success", "data": 123}),
-            UnexpectedItem(status_code=200, id="unexpected", item={"externalId": "unexpected", "data": 999}),
-            MissingItem(status_code=200, id="missing"),
-            FailedItem(status_code=400, id="fail", error="Item failed"),
+            SuccessResponseItems(
+                status_code=200, ids=["success"], body='{"items":[{"externalId":"success","data":123}]}'
+            ),
+            FailedResponseItems(
+                status_code=400,
+                ids=["fail"],
+                error=ErrorDetails(message="Item failed", code=400),
+                body='{"error":"Item failed"}',
+            ),
         ]
-        assert len(rsps.calls) == 5  # Three requests made
-        first, second, third, fourth, fifth = rsps.calls
+        assert len(rsps.calls) == 3  # Three requests made
+        first, second, third = rsps.calls
         # First call will fail, and split into 1 item + 2 items
         assert json.loads(first.request.content)["items"] == [
             {"externalId": "success"},
-            {"externalId": "missing"},
             {"externalId": "fail"},
         ]
         # Second succeeds with 1 item.
         assert json.loads(second.request.content)["items"] == [{"externalId": "success"}]
-        # Third fails with two items, and splits into 1 + 1
-        assert json.loads(third.request.content)["items"] == [{"externalId": "missing"}, {"externalId": "fail"}]
-        # Fourth succeeds with 1 item.
-        assert json.loads(fourth.request.content)["items"] == [{"externalId": "missing"}]
-        # Fifth fails with 1 item.
-        assert json.loads(fifth.request.content)["items"] == [{"externalId": "fail"}]
+        # Third fails with 1 item.
+        assert json.loads(third.request.content)["items"] == [{"externalId": "fail"}]
 
     def test_request_all_item_fail(self, http_client: HTTPClient, rsps: respx.MockRouter) -> None:
         rsps.post("https://example.com/api/resource").respond(
             json={"error": "Unauthorized"},
             status_code=401,
         )
-        items = [{"name": "item1", "id": 1}, {"name": "item2", "id": 2}]
+        items = [MyRequestItemID({"name": "item1", "id": 1}), MyRequestItemID({"name": "item2", "id": 2})]
         results = http_client.request(
             ItemsRequest[int](
                 endpoint_url="https://example.com/api/resource",
                 method="POST",
                 items=items,
-                as_id=lambda item: item["id"],
             )
         )
         assert results == [
-            FailedItem(status_code=401, id=1, error="Unauthorized"),
-            FailedItem(status_code=401, id=2, error="Unauthorized"),
+            FailedResponseItems(
+                status_code=401,
+                ids=[1, 2],
+                error=ErrorDetails(message="Unauthorized", code=401),
+                body='{"error":"Unauthorized"}',
+            ),
         ]
-
         assert len(rsps.calls) == 1  # Only one request made
 
-    def test_bad_request_items(self, http_client: HTTPClient, rsps: respx.MockRouter) -> None:
-        # Test with non-serializable item
-        bad_items = [
-            {"id": 1},
-            {"externalId": "duplicate"},
-            {"externalId": "duplicate"},
-        ]  # Duplicate externalId will cause issue
-        rsps.post("https://example.com/api/resource").respond(
-            json={"items": [{"externalId": "duplicate", "data": 123}]},
-            status_code=200,
-        )
-
-        results = http_client.request(
-            ItemsRequest[str](
-                endpoint_url="https://example.com/api/resource",
-                method="POST",
-                items=bad_items,
-                as_id=lambda item: item["externalId"],  # KeyError will occur here
-            )
-        )
-        assert results == [
-            UnknownRequestItem(error="Error extracting ID: 'externalId'", item={"id": 1}),
-            FailedRequestItem(id="duplicate", error="Duplicate item ID: 'duplicate'"),
-            SuccessItem(status_code=200, id="duplicate", item={"externalId": "duplicate", "data": 123}),
-        ]
-
     def test_request_no_items_response(self, http_client: HTTPClient, rsps: respx.MockRouter) -> None:
         rsps.post("https://example.com/api/resource/delete").respond(status_code=200)
-        items = [{"id": 1}, {"id": 2}]
+        items = [MyRequestItemID({"id": 1}), MyRequestItemID({"id": 2})]
         results = http_client.request(
             ItemsRequest[int](
                 endpoint_url="https://example.com/api/resource/delete",
                 method="POST",
                 items=items,
-                as_id=lambda item: item["id"],
             )
         )
         assert results == [
-            SuccessItem(status_code=200, id=1),
-            SuccessItem(status_code=200, id=2),
-        ]
-
-    def test_response_unknown_id(self, http_client: HTTPClient, rsps: respx.MockRouter) -> None:
-        rsps.post("https://example.com/api/resource").respond(
-            json={"items": [{"uid": "a", "data": 1}, {"uid": "b", "data": 2}]},
-            status_code=200,
-        )
-        items = [{"name": "itemA", "id": 1}, {"name": "itemB", "id": 2}]
-        results = http_client.request(
-            ItemsRequest[int](
-                endpoint_url="https://example.com/api/resource",
-                method="POST",
-                items=items,
-                as_id=lambda item: item["id"],
-            )
-        )
-        assert results == [
-            UnknownResponseItem(status_code=200, item={"uid": "a", "data": 1}, error="Error extracting ID: 'id'"),
-            UnknownResponseItem(status_code=200, item={"uid": "b", "data": 2}, error="Error extracting ID: 'id'"),
-            MissingItem(status_code=200, id=1),
-            MissingItem(status_code=200, id=2),
+            SuccessResponseItems(status_code=200, ids=[1, 2], body=""),
         ]
 
     def test_timeout_error(self, http_client_one_retry: HTTPClient, rsps: respx.MockRouter) -> None:
@@ -393,12 +360,11 @@ class TestHTTPClientItemRequests:
                 ItemsRequest[int](
                     endpoint_url="https://example.com/api/resource",
                     method="POST",
-                    items=[{"id": 1}],
-                    as_id=lambda item: item["id"],
+                    items=[MyRequestItemID({"id": 1})],
                 )
             )
         assert results == [
-            FailedRequestItem(id=1, error="RequestException after 1 attempts (read error): Simulated timeout error")
+            FailedRequestItems(ids=[1], error="RequestException after 1 attempts (read error): Simulated timeout error")
         ]
 
     @pytest.mark.usefixtures("disable_gzip")
@@ -413,18 +379,17 @@ class TestHTTPClientItemRequests:
                 ItemsRequest[int](
                     endpoint_url="https://example.com/api/resource",
                     method="POST",
-                    items=[{"id": i} for i in range(1000)],
-                    as_id=lambda item: item["id"],
+                    items=[MyRequestItemID({"id": i}) for i in range(1000)],
                     max_failures_before_abort=5,
                 )
             )
         assert len(rsps.calls) == 5
         actual_items_per_request = [len(json.loads(call.request.content)["items"]) for call in rsps.calls]
         assert actual_items_per_request == [1000, 500, 500, 250, 250]  # Splits in half each time
-        failures = Counter([type(results) for results in results])
+        failures = Counter([type(result) for result in results if isinstance(result, ItemMessage) for _ in result.ids])
         assert failures == {
-            FailedItem: 250,  # 250 items keeps the original error message.
-            FailedRequestItem: 750,  # 750 items get the early abort message.
+            FailedResponseItems: 250,  # 250 items keeps the original error message.
+            FailedRequestItems: 750,  # 750 items get the early abort message.
         }
 
     @pytest.mark.usefixtures("disable_gzip")
@@ -438,13 +403,12 @@ class TestHTTPClientItemRequests:
             ItemsRequest[int](
                 endpoint_url="https://example.com/api/resource",
                 method="POST",
-                items=[{"id": i} for i in range(1000)],
-                as_id=lambda item: item["id"],
+                items=[MyRequestItemID({"id": i}) for i in range(1000)],
                 max_failures_before_abort=1,
             )
         )
         actual_failure_types = Counter([type(results) for results in results])
-        assert actual_failure_types == {FailedItem: 1000}
+        assert actual_failure_types == {FailedResponseItems: 1}
         assert len(rsps.calls) == 1
         actual_items_per_request = [len(json.loads(call.request.content)["items"]) for call in rsps.calls]
         assert actual_items_per_request == [1000]
@@ -460,13 +424,14 @@ class TestHTTPClientItemRequests:
             ItemsRequest[int](
                 endpoint_url="https://example.com/api/resource",
                 method="POST",
-                items=[{"id": i} for i in range(1000)],
-                as_id=lambda item: item["id"],
+                items=[MyRequestItemID({"id": i}) for i in range(1000)],
                 max_failures_before_abort=2,
             )
         )
-        actual_failure_types = Counter([type(results) for results in results])
-        assert actual_failure_types == {FailedItem: 500, FailedRequestItem: 500}
+        actual_failure_types = Counter(
+            [type(results) for results in results if isinstance(results, ItemMessage) for _ in results.ids]
+        )
+        assert actual_failure_types == {FailedResponseItems: 500, FailedRequestItems: 500}
         assert len(rsps.calls) == 2
         actual_items_per_request = [len(json.loads(call.request.content)["items"]) for call in rsps.calls]
         assert actual_items_per_request == [1000, 500]
@@ -482,13 +447,14 @@ class TestHTTPClientItemRequests:
             ItemsRequest[int](
                 endpoint_url="https://example.com/api/resource",
                 method="POST",
-                items=[{"id": i} for i in range(100)],
-                as_id=lambda item: item["id"],
+                items=[MyRequestItemID({"id": i}) for i in range(100)],
                 max_failures_before_abort=-1,  # Never abort
             )
         )
-        actual_failure_types = Counter([type(results) for results in results])
-        assert actual_failure_types == {FailedItem: 100}
+        actual_failure_types = Counter(
+            [type(results) for results in results if isinstance(results, ItemMessage) for _ in results.ids]
+        )
+        assert actual_failure_types == {FailedResponseItems: 100}
         assert len(rsps.calls) == 199
 
     @pytest.mark.usefixtures("disable_gzip")
@@ -517,13 +483,14 @@ class TestHTTPClientItemRequests:
                 ItemsRequest[int](
                     endpoint_url="https://example.com/api/resource",
                     method="POST",
-                    items=[{"id": i} for i in range(1000)],
-                    as_id=lambda item: item["id"],
+                    items=[MyRequestItemID({"id": i}) for i in range(1000)],
                     max_failures_before_abort=30,
                 )
             )
-        failures = Counter([type(results) for results in results])
-        assert failures == {FailedItem: 3, SuccessItem: 997}
+        failures = Counter(
+            [type(results) for results in results if isinstance(results, ItemMessage) for _ in results.ids]
+        )
+        assert failures == {FailedResponseItems: 3, SuccessResponseItems: 997}
 
 
 class TestHTTPMessage:
