@@ -61,7 +61,6 @@ from cognite.client.testing import CogniteClientMock
 from cognite.client.utils.useful_types import SequenceNotStr
 from questionary import Choice
 
-from cognite_toolkit._cdf_tk._parameters.get_type_hints import _TypeHints
 from cognite_toolkit._cdf_tk.client.data_classes.location_filters import (
     LocationFilter,
     LocationFilterScene,
@@ -69,7 +68,9 @@ from cognite_toolkit._cdf_tk.client.data_classes.location_filters import (
 from cognite_toolkit._cdf_tk.client.data_classes.sequences import ToolkitSequenceRows
 from cognite_toolkit._cdf_tk.constants import MODULES
 from cognite_toolkit._cdf_tk.utils import load_yaml_inject_variables, read_yaml_file
+from cognite_toolkit._cdf_tk.utils.http_client._data_classes import T_COVARIANT_ID, RequestItem
 from tests.data import COMPLETE_ORG
+from tests.test_unit.get_type_hints import _TypeHints
 
 UNION_TYPES = {typing.Union, UnionType}
 
@@ -401,6 +402,17 @@ class FakeCogniteResourceGenerator:
                 [self.create_value(first_not_none) for _ in range(self._random.randint(1, 3))],
                 cursor=self.create_value(str),
             )
+        elif container_type is RequestItem:
+
+            @dataclasses.dataclass
+            class FakeRequestItem(RequestItem):
+                def dump(self) -> dict[str, Any]:
+                    return {"key": "value"}
+
+                def as_id(self) -> str:
+                    return "identifier"
+
+            return FakeRequestItem()
 
         if var_name == "external_id" and type_ is str:
             return self._random_string(50, sample_from=string.ascii_uppercase + string.digits)
@@ -449,11 +461,11 @@ class FakeCogniteResourceGenerator:
             return self.create_instance(selected)
         elif isinstance(type_, enum.EnumMeta):
             return self._random.choice(list(type_))
-        elif isinstance(type_, TypeVar):
+        elif isinstance(type_, TypeVar) and type_.__bound__ is not None:
             return self.create_value(type_.__bound__)
         elif inspect.isclass(type_) and issubclass(type_, CogniteResourceList):
             return type_([self.create_value(type_._RESOURCE) for _ in range(self._random.randint(1, 3))])
-        elif type_ is Hashable:
+        elif type_ is Hashable or type_ is T_COVARIANT_ID:
             return "my_hashable"
         elif inspect.isclass(type_):
             return self.create_instance(type_)
